@@ -4,11 +4,11 @@ if int(os.environ.get('PROFILE_MEM', '0')):
     from ul_api_utils.debug.malloc import trace_malloc
 
     with trace_malloc(show_all=True):
-        from .conf import sdk, db_config
+        from example.conf import sdk, db_config
 
         flask_app = sdk.init_with_flask(__name__, db_config=db_config)
 else:
-    from .conf import sdk, db_config
+    from example.conf import sdk, db_config
 
     flask_app = sdk.init_with_flask(__name__, db_config=db_config)
 
