@@ -1,23 +1,28 @@
-from datetime import datetime, timedelta
+import logging
 from time import sleep
-from typing import List, Optional, Tuple
+from datetime import datetime, timedelta
+from typing import List, Optional, Tuple, Any
 
-from ul_db_utils.modules.db import db
+from flask_socketio import emit
+
+from ul_api_utils.resources.sockets import socket_io
 from flask import jsonify
 from pydantic import BaseModel
-
+from ul_db_utils.modules.postgres_modules.db import db
 from werkzeug import Response as BaseResponse
+from flask import request
+from example.conf import sdk
+from example.permissions import SOME_PERMISSION, SOME_PERMISSION2
 from ul_api_utils.api_resource.api_request import ApiRequestQuery
 from ul_api_utils.api_resource.api_resource import ApiResource
 from ul_api_utils.api_resource.api_resource_config import ApiResourceConfig
-from ul_api_utils.api_resource.api_response import FileApiResponse, JsonApiResponsePayload, JsonApiResponse, AnyJsonApiResponse, ProxyJsonApiResponse, HtmlApiResponse
+from ul_api_utils.api_resource.api_response import FileApiResponse, JsonApiResponsePayload, JsonApiResponse, \
+    AnyJsonApiResponse, ProxyJsonApiResponse, HtmlApiResponse, EmptyJsonApiResponse
 from ul_api_utils.errors import Server5XXInternalApiError, NoResultFoundApiError, Client4XXInternalApiError
 from ul_api_utils.internal_api.internal_api import InternalApi
 from ul_api_utils.utils.api_encoding import ApiEncoding
 from ul_api_utils.resources.health_check.health_check import HealthCheckContext
 from ul_api_utils.validators.custom_fields import QueryParamsSeparatedList
-from example.conf import sdk
-from example.permissions import SOME_PERMISSION, SOME_PERMISSION2
 
 internal_api = InternalApi(
     entry_point='http://localhost:5000',
@@ -47,11 +52,30 @@ class SomeBody(BaseModel):
 
 class Some2Query(ApiRequestQuery):
     sleep: float = 0.4
-    some: QueryParamsSeparatedList[str]
+    some: QueryParamsSeparatedList[str] = ''        # type: ignore
+
+
+logger = logging.getLogger(__name__)
 
 
 @sdk.rest_api('POST', '/example-resource-simple', access=sdk.ACCESS_PUBLIC)
+@sdk.cache_api('REFRESH', 'example')
 def some5(api_resource: ApiResource, body: SomeBody) -> JsonApiResponse[List[RespObject]]:
+    """
+    The API returns user details for a given username.
+    The API can be invoked using *curl* like below:
+    ```
+    curl --header accept: application/json -u username:password http://localhost:8080/api/v2/user/jhondoe
+    ```
+    **Sample Output**
+    ```
+    {
+      "id": 2,
+      "username": "jhondoe"
+      "email": "jhon.doe@mail.com"
+    }
+    ```
+    """
     return api_resource.response_ok([
         RespObject(now=datetime.now() + timedelta(seconds=body.seconds)),
         RespObject(now=datetime.now() + timedelta(seconds=body.seconds * 2)),
@@ -63,6 +87,7 @@ class Some7Query(ApiRequestQuery):
 
 
 @sdk.rest_api('POST', '/example-resource-simple-any', access=sdk.ACCESS_PUBLIC)
+@sdk.cache_api('READ', 'example')
 def some9(api_resource: ApiResource) -> AnyJsonApiResponse:
     return api_resource.response_ok([RespObject(now=datetime.now(), notes="some9")], 1)
 
@@ -91,7 +116,7 @@ def some7(api_resource: ApiResource, query: Some7Query) -> JsonApiResponse[List[
 
 @sdk.rest_api('GET', '/example-resource-simple-proxy', access=sdk.ACCESS_PUBLIC)
 def some6(api_resource: ApiResource) -> ProxyJsonApiResponse[List[RespObject]]:  # type: ignore
-    res = internal_api.request_post('/example-resource-simple', json={'seconds': 123}).typed(List[RespObject]).check()
+    res = internal_api.request_post('/example-resource-simple-any', json={'seconds': 123}).typed(List[RespObject]).check()
     assert res.payload[0].now is not None
 
     return api_resource.response_proxy(res)
@@ -151,6 +176,7 @@ def health_check(context: HealthCheckContext) -> None:
 
 
 @sdk.rest_api('GET', '/example-resource', access=sdk.ACCESS_PUBLIC)
+@sdk.cache_api('READ', ('example', 'resource', 'simple'))
 def some3(api_resource: ApiResource) -> JsonApiResponse[RespObject]:
     api_resource.logger.info('some 1')
     sleep(0.1)
@@ -182,6 +208,7 @@ def some2html(api_resource: ApiResource, query: Some2Query) -> HtmlApiResponse:
 
 
 @sdk.rest_api('GET', '/example-resource-for-loong-sleep', access=sdk.ACCESS_PUBLIC)
+@sdk.cache_api('READ', ('example', 'resource', 'simple'))
 def some2(api_resource: ApiResource, query: Some2Query) -> JsonApiResponse[RespObject]:
     sess = db.session()
     sess.execute('SELECT * FROM information_schema.tables LIMIT 3;')
@@ -191,6 +218,7 @@ def some2(api_resource: ApiResource, query: Some2Query) -> JsonApiResponse[RespO
 
 
 @sdk.rest_api('GET', '/example-resource-empty', access=sdk.ACCESS_PUBLIC)
+@sdk.cache_api('READ', ('example', 'resource'))
 def some1(api_resource: ApiResource) -> JsonApiResponse[RespObject]:
     return api_resource.response_ok(RespObject(now=datetime.now()))
 
@@ -204,6 +232,17 @@ def override(api_resource: ApiResource) -> JsonApiResponse[RespObject]:
     return api_resource.response_ok(RespObject(now=datetime.now()))
 
 
+@sdk.rest_api('GET', '/example-resource-empty', access=sdk.ACCESS_PUBLIC)
+def empty_resp_test(api_resource: ApiResource) -> EmptyJsonApiResponse:
+    return api_resource.response_empty_ok()
+
+
+@sdk.rest_api('POST', '/example-resource-empty-check', access=sdk.ACCESS_PUBLIC)
+def empty_resp_req_test(api_resource: ApiResource) -> EmptyJsonApiResponse:
+    internal_api.request_get('/example-resource-empty').check()
+    return api_resource.response_empty_ok()
+
+
 @sdk.rest_api('POST', '/example-resource-empty-gzip', access=sdk.ACCESS_PUBLIC)
 def some12(api_resource: ApiResource, body: List[SomeBody]) -> JsonApiResponse[RespObject]:
     sess = db.session()
@@ -213,6 +252,7 @@ def some12(api_resource: ApiResource, body: List[SomeBody]) -> JsonApiResponse[R
 
 
 @sdk.html_view(('GET', 'POST'), '/', access=sdk.ACCESS_PUBLIC)
+@sdk.cache_api('READ', 'some')
 def view_home(api_resource: ApiResource, body: Optional[List[SomeBody]]) -> HtmlApiResponse:
     sleep(0.02)
     sess = db.session()
@@ -281,3 +321,21 @@ def somefile(api_resource: ApiResource) -> FileApiResponse:
         as_attachment=True,
         attachment_filename='import_devices_log.txt',
     )
+
+
+@socket_io.on('message')
+def handle(data: Any, message: Any) -> None:
+    sid = request.sid
+    emit('message', {"data1": 1, "data": 2}, room=sid)
+
+
+@socket_io.event
+def connect():
+    logger.info("Connected!")
+    logger.info("SESSION INFO: " + str(request.sid))
+
+
+@socket_io.event('disconnect')
+def disconnect():
+    logger.info("Client disconnected!")
+    logger.info("SESSION INFO: " + str(request.sid))
