@@ -25,6 +25,7 @@ class CmdStart(Cmd):
     debug: bool
     max_requests: int
     max_requests_jitter: int
+    worker_class: str
 
     @property
     def app_rel_dir(self) -> str:
@@ -49,6 +50,7 @@ class CmdStart(Cmd):
         parser.add_argument('--debug', dest='debug', type=arg_str2bool, default=False, required=False)
         parser.add_argument('--max-requests', dest='max_requests', type=int, default=1000, required=False)
         parser.add_argument('--max-requests-jitter', dest='max_requests_jitter', type=int, default=50, required=False)
+        parser.add_argument('--worker-class', dest='worker_class', type=str, default='sync', required=False)
 
     def run(self) -> None:
         env = os.environ.copy()
@@ -69,6 +71,7 @@ class CmdStart(Cmd):
         args = [
             f'-n={name}',
             f'-w={APPLICATION_GUNICORN_WORKERS}',
+            f'--worker-class={self.worker_class}',
             f'-b=0.0.0.0:{self.port}',
             f'--config={conf}',
             '--log-level=INFO',
