@@ -206,6 +206,8 @@ class ApiSdk:
                 async_mode=SocketAsyncModesEnum.GEVENT.value,
                 channel=self.config.sockets_config.channel,
                 cors_allowed_origins=self.config.sockets_config.cors_allowed_origins,
+                logger=self.config.sockets_config.logs_enabled,
+                engineio_logger=self.config.sockets_config.engineio_logs_enabled,
             )
 
         return self._flask_app
