@@ -40,6 +40,8 @@ class SocketIOConfig(BaseModel):
     message_queue: str
     channel: Optional[str] = "flask-socketio"
     cors_allowed_origins: Optional[str] = "*"
+    logs_enabled: Optional[bool] = False
+    engineio_logs_enabled: Optional[bool] = False
 
 
 class ApiSdkConfig(BaseModel):
