@@ -309,7 +309,7 @@ class SwaggerRequestType(SwaggerModel):
         if self.summary is not None:
             request_type_entry += f"\n  summary: {self.summary}"
         if self.description is not None:
-            request_type_entry += f"\n  description: {self.description}"
+            request_type_entry += f"\n  description: |\n{self.TAB}{self.TAB}{self.description}"
         request_type_entry = self.indent(request_type_entry, 2 * self.TAB)
 
         file.write(request_type_entry)
